<?php

require($PHP_PATH."Template.inc");
require($PHP_PATH."UserManagement.inc");

/**
 * Copyright (C) 2002 Oliver Hitz <oliver@net-track.ch>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA  02111-1307, USA.
 */

class AuthManager {
  var $um;
  
  var $version = "0.1";

  function AuthManager($dir = "") {
    global $HTTP_SERVER_VARS;

    if ($dir != "" && $dir[0] == "/") {
      $dir = realpath($dir);
    } else {
      $dir = dirname(realpath($HTTP_SERVER_VARS["SCRIPT_FILENAME"]));
    }

    $userfile = "";
    $groupfile = "";

    while ($userfile == "" || $groupfile == "") {
      if (file_exists($dir."/.htaccess")) {
	$filename = $dir."/.htaccess";
	
	if (is_readable($filename)) {
	  $f = file($filename);
	  
	  while (list($ln, $l) = each($f)) {
	    if ($userfile == "" && eregi("authuserfile[\ \t]+([^\ \t\n]+)[\ \t\n]", $l, $m)) {
	      $userfile = $m[1];
	    }
	    if ($groupfile == "" && eregi("authgroupfile[\ \t]+([^\ \t\n]+)[\ \t\n]", $l, $m)) {
	      $groupfile = $m[1];
	    }
	  }
	} else {
	  die($filename." is not readable.");
	}
      }
      if ($dir == "/") {
	break;
      } else {
	$dir = realpath($dir."/..");
      }
    }

    if ($userfile == "") {
      die("No users file found in your .htaccess files.");
    }
    if ($groupfile == "") {
      die("No groups file found in your .htaccess files.");
    }

    $this->um = new UserManagement($userfile, $groupfile);

    $this->main();
  }

  function main() {
    global $HTTP_GET_VARS;
    global $HTTP_POST_VARS;

    if (isset($HTTP_GET_VARS["cmd"])) {
      $cmd = $HTTP_GET_VARS["cmd"];
    } else if (isset($HTTP_POST_VARS["cmd"])) {
      $cmd = $HTTP_POST_VARS["cmd"];
    } else {
      $cmd = "list";
    }

    switch ($cmd) {
    case "about":
      $this->showAbout();
      break;
    case "list":
      $this->showList();
      break;

    case "edituser":
      $login = $HTTP_GET_VARS["user"];
      $this->um->getUserGroups($login);
      $this->editUser($login, $groups, $login);
      break;
    case "newuser":
      $this->newUser();
      break;
    case "saveuser":
      $this->saveUser();
      break;
    case "deleteuser":
      $this->deleteUser();
      break;

    case "editgroup":
      $group = $HTTP_GET_VARS["group"];
      $this->editGroup($group, $group);
      break;
    case "newgroup":
      $this->newGroup();
      break;
    case "savegroup":
      $this->saveGroup();
      break;
    case "deletegroup":
      $this->deleteGroup();
      break;

    case "addmember":
      $this->addMember();
      break;
    case "deletemember":
      $this->deleteMember();
      break;
    default:
      die("Unknown command $cmd.");
    }
  }

  function addMember() {
    global $HTTP_GET_VARS;
    $this->um->addMember($HTTP_GET_VARS["user"], $HTTP_GET_VARS["group"]);
    $this->um->close();
    $this->showList();
  }

  function deleteMember() {
    global $HTTP_GET_VARS;
    $this->um->deleteMember($HTTP_GET_VARS["user"], $HTTP_GET_VARS["group"]);
    $this->um->close();
    $this->showList();
  }

  function showAbout() {
    $tpl = new Template("templates/main.ihtml");
    $tpl->setVar("main", "AuthManager ".$this->version);
    print $tpl->toString();
  }

  function showList() {
    $t = new Template("templates/list.ihtml");

    $users = $this->um->getUsers();
    sort($users);

    $groups = $this->um->getGroups();
    sort($groups);

    $t->setVar("groupnumber", 3*count($groups));

    for ($i = 0; $i < count($groups); $i++) {
      $t->gotoNext("group");
      $t->setVar("group.name", $groups[$i]);
    }

    for ($i = 0; $i < count($users); $i++) {
      $t->gotoNext("user");
      $t->setVar("user.login", $users[$i]);

      for ($j = 0; $j < count($groups); $j++) {
	$t->gotoNext("user.ugroup");

	if ($this->um->isMember($users[$i], $groups[$j])) {
	  $t->setVar("user.ugroup.status", 1);
	} else {
	  $t->setVar("user.ugroup.status", 0);
	}
	$t->setVar("user.ugroup.name", $groups[$j]);
	$t->setVar("user.ugroup.user", $users[$i]);
      }
    }

    $tpl = new Template("templates/main.ihtml");
    $tpl->setVar("main", $t->toString());
    print $tpl->toString();
  }


  function newUser() {
    $this->editUser("", array(), "");
  }

  function editUser($login = "", $groups = "", $oldlogin) {
    global $HTTP_GET_VARS;

    if ($groups == "") {
      $groups = array();
    }

    $t = new Template("templates/edituser.ihtml");
    $t->setVar("login", $login);
    $t->setVar("oldlogin", $oldlogin);

    $g = $this->um->getGroups();
    for ($i = 0; $i < count($g); $i++) {
      $t->gotoNext("group");
      $t->setVar("group.name", $g[$i]);

      if ($this->um->isMember($login, $g[$i])) {
	$t->setVar("group.active", 1);
      } else {
	$t->setVar("group.active", 0);
      }
    }
    
    $tpl = new Template("templates/main.ihtml");
    $tpl->setVar("main", $t->toString());
    print $tpl->toString();
  }

  function saveUser() {
    global $HTTP_POST_VARS;

    if (isset($HTTP_POST_VARS["cancel"])) {
      $this->showList();
      return;
    }

    $login = trim($HTTP_POST_VARS["login"]);
    $oldlogin = trim($HTTP_POST_VARS["oldlogin"]);
    $password = trim($HTTP_POST_VARS["password"]);
 
    $groups = array();
    $g = $this->um->getGroups();
    for ($i = 0; $i < count($g); $i++) {
      if (isset($HTTP_POST_VARS["group"][$g[$i]])) {
	$groups[] = $g[$i];
      }
    }

    if ($login == "" || ($oldlogin == "" && $password == "")) {
      $this->editUser($login, $groups, $oldlogin);
      return;
    }

    if ($oldlogin != "") {
      $oldpassword = $this->um->getPassword($oldlogin);
      $this->um->deleteUser($oldlogin);
    }
    $this->um->deleteUser($login);
    if ($password != "") {
      $this->um->addUser($login, crypt($password));
    } else {
      $this->um->addUser($login, $oldpassword);
    }

    for ($i = 0; $i < count($groups); $i++) {
      $this->um->addMember($login, $groups[$i]);
    }

    $this->um->close();
    $this->showList();
  }

  function deleteUser() {
    global $HTTP_GET_VARS;
    $this->um->deleteUser($HTTP_GET_VARS["user"]);
    $this->um->close();
    $this->showList();
  }

  function newGroup() {
    $this->editGroup("", "");
  }

  function editGroup($group, $oldgroup) {
    $t = new Template("templates/editgroup.ihtml");
    $t->setVar("group", $group);
    $t->setVar("oldgroup", $oldgroup);

    $tpl = new Template("templates/main.ihtml");
    $tpl->setVar("main", $t->toString());
    print $tpl->toString();
  }

  function saveGroup() {
    global $HTTP_POST_VARS;

    if (isset($HTTP_POST_VARS["cancel"])) {
      $this->showList();
      return;
    }

    $group = trim($HTTP_POST_VARS["group"]);
    $oldgroup = trim($HTTP_POST_VARS["oldgroup"]);
 
    if ($group == "") {
      $this->editGroup($group, $oldgroup);
      return;
    }

    if ($oldgroup != "") {
      $this->um->deleteGroup($oldgroup);
    }
    $this->um->deleteGroup($group);
    $this->um->addGroup($group);

    $this->um->close();

    $this->showList();
  }

  function deleteGroup() {
    global $HTTP_GET_VARS;
    $this->um->deleteGroup($HTTP_GET_VARS["group"]);
    $this->um->close();
    $this->showList();
  }
}

?>
