<?php

/**
 * Copyright (C) 2002 Oliver Hitz <oliver@net-track.ch>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA  02111-1307, USA.
 */

class Template {

  var $content;

  var $filename = "";

  function Template($filename) {
    $this->filename = $filename;

    $this->content = array();
    $this->content["name"] = "main";
    $this->content["body"] = $this->loadfile($this->filename);
    $this->content["blocks"] = array();
    $this->content["values"] = array();

    $this->parse(&$this->content);
  }

  function showStructure() {
    print "<ul>";
    $this->_showStructure(&$this->content);
    print "</ul>";
  }

  function _showStructure(&$content) {
    print "<li>name: ".$content["name"]."</li>";
    print "<li>body: ".htmlentities($content["body"])."</li>";
    print "<li>blocks:";
    print "<ul>";
    reset ($content["blocks"]);
    while (list($blockname, $block) = each($content["blocks"])) {
      for ($i = 0; $i < count($block); $i++) {
	$this->_showStructure($block[$i]);
      }
    }
    print "</ul>";
    print "</li>";
  }

  function parse(&$content) {
    while (eregi("<!--\ startBlock\(([^)]+)\)\ -->", $content["body"], $m)) {
      $name = $m[1];
      $block = array();
      $block["name"] = $name;
      $block["blocks"] = array();
      $block["used"] = 0;
      $block["values"] = array();
      $reg = "<!--\ startBlock\(".$name."\)\ -->(.*)<!--\ endBlock\(".$name."\)\ -->";
      if (!eregi($reg, $content["body"], $m)) {
	$this->error("block `".$name."' does not have startBlock() AND endBlock().");
      }
      $block["body"] = $m[1];
      $content["body"] = eregi_replace($reg, "{".$name."}", $content["body"]);
      $content["blocks"][$name] = array();
      $this->parse(&$block);
      $content["blocks"][$name][0] = $block;
    }
  }

  function toString() {
    return $this->_toString(&$this->content, 0);
  }

  function _toString(&$content, $count) {
    global $HTTP_SERVER_VARS;

    $txt = $content["body"];

    reset($content["values"]);
    while (list($key, $val) = each($content["values"])) {
      if (!is_numeric($key)) {
	$txt = eregi_replace("{".$key."}", $val, $txt);
      }
    }

    $txt = eregi_replace("{sid}", session_id(), $txt);
    $txt = eregi_replace("{script}", $HTTP_SERVER_VARS["SCRIPT_NAME"], $txt);

    while (eregi("{url:([^}]+)}", $txt, $m)) {
      $txt = eregi_replace("{url:".$m[1]."}", urlencode($content["values"][$m[1]]), $txt);
    }

    while (eregi("{format:([^:]+):([^}]+)}", $txt, $m)) {
      $txt = eregi_replace("{format:".$m[1].":".$m[2]."}", sprintf($m[2], $content["values"][$m[1]]), $txt);
    }

    while (eregi("{date:([^:]+):([^}]+)}", $txt, $m)) {
      $txt = eregi_replace("{date:".$m[1].":".$m[2]."}", date($m[2], $content["values"][$m[1]]), $txt);
    }

    while (eregi("{ifEq:([^:]+):([^:]+):([^}]+)}", $txt, $m)) {
      if ($content["values"][$m[1]] == $m[2]) {
	$txt = eregi_replace("{ifEq:".$m[1].":".$m[2].":([^}]+)}", "\\1", $txt);
      } else {
	$txt = eregi_replace("{ifEq:".$m[1].":".$m[2].":([^}]+)}", "", $txt);
      }
    }

    while (eregi("{ifNe:([^:]+):([^:]*):([^}]+)}", $txt, $m)) {
      if ($content["values"][$m[1]] != $m[2]) {
	$txt = eregi_replace("{ifNe:".$m[1].":".$m[2].":([^}]+)}", "\\1", $txt);
      } else {
	$txt = eregi_replace("{ifNe:".$m[1].":".$m[2].":([^}]+)}", "", $txt);
      }
    }

    while (eregi("{ifSet:([^:]+):([^}]+)}", $txt, $m)) {
      if ($content["values"][$m[1]] != "" && $content["values"][$m[1]] != "0") {
	$txt = eregi_replace("{ifSet:".$m[1].":([^}]+)}", "\\1", $txt);
      } else {
	$txt = eregi_replace("{ifSet:".$m[1].":([^}]+)}", "", $txt);
      }
    }

    while (eregi("{ifNotSet:([^:]+):([^}]+)}", $txt, $m)) {
      if ($content["values"][$m[1]] == "" || $content["values"][$m[1]] == "0") {
	$txt = eregi_replace("{ifNotSet:".$m[1].":([^}]+)}", "\\1", $txt);
      } else {
	$txt = eregi_replace("{ifNotSet:".$m[1].":([^}]+)}", "", $txt);
      }
    }
    
    if (eregi("{ifNotFirst:([^}]+)}", $txt, $m)) {
      if ($count == 0) {
	$txt = eregi_replace("{ifNotFirst:".$m[1]."}", "", $txt);
      } else {
	$txt = eregi_replace("{ifNotFirst:".$m[1]."}", $m[1], $txt);
      }
    }

    if (eregi("{ifOddPosition:([^}]+)}", $txt, $m)) {
      if ($count % 2 == 1) {
	$txt = eregi_replace("{ifOddPosition:".$m[1]."}", $m[1], $txt);
      } else {
	$txt = eregi_replace("{ifOddPosition:".$m[1]."}", "", $txt);
      }
    }
    if (eregi("{ifEvenPosition:([^}]+)}", $txt, $m)) {
      if ($count % 2 == 0) {
	$txt = eregi_replace("{ifEvenPosition:".$m[1]."}", $m[1], $txt);
      } else {
	$txt = eregi_replace("{ifEvenPosition:".$m[1]."}", "", $txt);
      }
    }

    $txt = eregi_replace("{count:1}", "".($count+1), $txt);
    $txt = eregi_replace("{count:0}", "".($count), $txt);

    reset ($content["blocks"]);
    while (list($blockname, $block) = each($content["blocks"])) {
      if (!is_numeric($blockname)) {
	$rep = "";
	for ($i = 0; $i < count($block); $i++) {
	  if ($block[$i]["used"]) {
	    $rep .= $this->_toString($block[$i], $i);
	  }
	}
	$txt = eregi_replace("{".$blockname."}", $rep, $txt);
      }
    }

    return $txt;
  }

  function setVar($variable, $value) {
    if (is_array($value)) {
      if ($variable == "") {
	$path = "";
      } else {
	$path = $variable.".";
      }
      reset($value);
      while (list($var, $val) = each($value)) {
	$this->_setVar($this->content, $path.$var, $val);
      }
    } else {
      $value = $value."";
      $this->_setVar($this->content, $variable, $value);
    }
  }

  function _setVar(&$content, $variable, $value) {
    if (eregi("([^\.]+)\.(.*)", $variable, $m)) {
      $parent = $m[1];
      $var    = $m[2];

      $content["used"] = 1;
      $i = count($content["blocks"][$parent])-1;
      if (isset($content["blocks"][$parent])) {
	$this->_setVar($content["blocks"][$parent][$i], $var, $value);
      }
    } else {
      $content["used"] = 1;
      $content["values"][$variable] = $value;
    }
  }

  function gotoNext($variable) {
    $this->_gotoNext(&$this->content, $variable);
  }

  function _gotoNext(&$content, $variable) {
    if (eregi("([^\.]+)\.(.*)", $variable, $m)) {
      $parent = $m[1];
      $var    = $m[2];

      $i = count($content["blocks"][$parent])-1;
      $this->_gotoNext(&$content["blocks"][$parent][$i], $var);
    } else {
      $i = count($content["blocks"][$variable]);
      if (count($content["blocks"][$variable][$i-1]["values"]) > 0) {
	$content["blocks"][$variable][$i] = $this->_copyStructure($content["blocks"][$variable][$i-1]);
      }
    }
  }

  function &_copyStructure($content) {
    $newcontent = array();
    $newcontent["name"] = $content["name"];
    $newcontent["body"] = $content["body"];
    $newcontent["values"] = array();
    $newcontent["blocks"] = array();
    $newcontent["used"] = 0;

    reset($content["blocks"]);
    while (list($key, $value) = each($content["blocks"])) {
      $newcontent["blocks"][$key] = array();
      $newcontent["blocks"][$key][0] = $this->_copyStructure($content["blocks"][$key][0]);
    }

    return $newcontent;
  }

  function loadfile($filename) {
    return implode("", file($filename));
  }

  function error($message) {
    die($this->filename.": ".$message);
  }
}
