<?php

/**
 * Copyright (C) 2002 Oliver Hitz <oliver@net-track.ch>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA  02111-1307, USA.
 */

class UserManagement {
  var $userfile = "";
  var $groupfile = "";

  var $users;
  var $groups;

  var $changed = 0;

  function UserManagement($userfile, $groupfile) {
    if (!file_exists($userfile) && !is_writable(dirname($userfile))) {
      die("File $userfile does not exist and the directory containing it is not writable.");
    }
    if (!is_readable($userfile)) {
      die("File $userfile is not readable.");
    }
    if (!file_exists($groupfile) && !is_writable(dirname($groupfile))) {
      die("File $groupfile does not exist and the directory containing it is not writable.");
    }
    if (!is_readable($groupfile)) {
      die("File $groupfile is not readable.");
    }

    $this->userfile = $userfile;
    $this->groupfile = $groupfile;

    $this->open();
  }

  function open() {
    $this->users = array();
    $this->groups = array();

    if (file_exists($this->userfile)) {
      $f = file($this->userfile);
      for ($ln = 0; $ln < count($f); $ln++) {
	$tok = explode(":", ereg_replace("^([^#])#.*$", "\\1", trim($f[$ln])));
	if (count($tok) >= 2) {
	  $this->users[] = array("username" => $tok[0], "password" => $tok[1], "groups" => array());
	}
      }
    }

    if (file_exists($this->groupfile)) {
      $groups = array();

      $f = file($this->groupfile);
      for ($i = 0; $i < count($f); $i++) {
	$tok = explode(":", ereg_replace("^([^#])#.*$", "\\1", trim($f[$i])));
	if (count($tok) >= 2) {
	  $u = explode(" ", $tok[1]);
	  $users = array();
	  for ($j = 0; $j < count($u); $j++) {
	    $users[$u[$j]] = 1;
	  }
	  $groups[$tok[0]] = $users;
	}
      }

      $this->groups = array_keys($groups);

      for ($i = 0; $i < count($this->users); $i++) {
	$user = $this->users[$i]["username"];
	for ($j = 0; $j < count($this->groups); $j++) {
	  $group = $this->groups[$j];
	  if (isset($groups[$group][$user])) {
	    $this->users[$i]["groups"][$group] = 1;
	  } else {
	    $this->users[$i]["groups"][$group] = 0;
	  }
	}
      }
    }
  }

  /**
   * Saves the new password and group files if necessary.
   */
  function close() {
    if ($this->changed != 0) {
      $usercontent = "";
      for ($i = 0; $i < count($this->users); $i++) {
	$usercontent .= sprintf("%s:%s\n", $this->users[$i]["username"], $this->users[$i]["password"]);
      }

      $groups = $this->getGroups();

      $groupcontent = "";
      for ($i = 0; $i < count($groups); $i++) {
	$users = array();
	for ($j = 0; $j < count($this->users); $j++) {
	  if ($this->users[$j]["groups"][$groups[$i]] == 1) {
	    $users[] = $this->users[$j]["username"];
	  }
	}
	$groupcontent .= sprintf("%s: %s\n", $groups[$i], implode(" ", $users));
      }

      $this->replaceFile($this->userfile, $usercontent);
      $this->replaceFile($this->groupfile, $groupcontent);
    }
  }

  function replaceFile($filename, $contents) {
    $fd = fopen($filename.".new", "w");
    if ($fd == false) {
      die("Unable to create new file $filename.new");
    }

    fwrite($fd, $contents);
    fclose($fd);

    if (false == rename($filename.".new", $filename)) {
      die("Unable to rename $filename.new to $filename");
    }
  }

  /**
   * Returns the names of all groups.
   *
   * @return Names of the groups.
   */
  function getGroups() {
    return $this->groups;
  }

  /**
   * Returns the names of all users.
   *
   * @return Names of the users.
   */
  function getUsers() {
    $users = array();
    for ($i = 0; $i < count($this->users); $i++) {
      $users[] = $this->users[$i]["username"];
    }
    return $users;
  }

  /**
   * Returns an array with the names of the groups a user is member of.
   *
   * @param $username Name of the user.
   * @return Array containing the names of the groups.
   */
  function getUserGroups($username) {
    $groupnames = $this->getGroups();
    for ($i = 0; $i < count($this->users); $i++) {
      if ($this->users[$i]["username"] == $username) {
	$groups = array();
	for ($j = 0; $j < count($groupnames); $j++) {
	  if ($this->users[$i]["groups"][$groupnames[$j]] == 1) {
	    $groups[] = $groupnames[$j];
	  }
	}
	return $groups;
      }
    }
    return array();
  }

  /**
   * Returns the crypted password of a user.
   *
   * @param $username The name of the user.
   * @return The crypted password.
   */
  function getPassword($username) {
    for ($i = 0; $i < count($this->users); $i++) {
      if ($this->users[$i]["username"] == $username) {
	return $this->users[$i]["password"];
      }
    }
    return "";
  }

  /**
   * Adds the group "groupname" to the list of groups.
   *
   * @param $groupname Name of the group to add.
   */
  function addGroup($groupname) {
    $this->changed = 1;
    $this->groups[] = $groupname;
    for ($i = 0; $i < count($this->users); $i++) {
      $this->users[$i]["groups"][$groupname] = 0;
    }
  }

  /**
   * Removes a group from the list of groups.
   *
   * @param $groupname Name of the group to remove.
   */
  function deleteGroup($groupname) {
    $this->changed = 1;

    $g = array();
    for ($i = 0; $i < count($this->groups); $i++) {
      if ($this->groups[$i] != $groupname) {
	$g[] = $this->groups[$i];
      }
    }
    $this->groups = $g;

    for ($i = 0; $i < count($this->users); $i++) {
      unset($this->users[$i]["groups"][$groupname]);
    }
  }

  /**
   * Deletes the user "username" from the list of users.
   *
   * @param $username Name of the user to delete.
   */
  function deleteUser($username) {
    $this->changed = 1;

    $u = array();
    for ($i = 0; $i < count($this->users); $i++) {
      if ($this->users[$i]["username"] != $username) {
	$u[] = $this->users[$i];
      }
    }
    $this->users = $u;
  }

  /**
   * Adds a user to the list of users.
   *
   * @param $username Name of the user to add.
   * @param $password Crypted password of the user.
   */
  function addUser($username, $password) {
    $this->changed = 1;
    $this->users[] = array("username" => $username, "password" => $password, "groups" => array());
  }

  /**
   * Adds a user to a group.
   *
   * @param $username Name of the user to add.
   * @param $groupname Name of the group.
   */
  function addMember($username, $groupname) {
    $this->changed = 1;
    for ($i = 0; $i < count($this->users); $i++) {
      if ($this->users[$i]["username"] == $username) {
	$this->users[$i]["groups"][$groupname] = 1;
	return;
      }
    }
  }

  /**
   * Removes a user from a group.
   *
   * @param $username Name of the user to remove.
   * @param $groupname Name of the group.
   */
  function deleteMember($username, $groupname) {
    $this->changed = 1;
    for ($i = 0; $i < count($this->users); $i++) {
      if ($this->users[$i]["username"] == $username) {
	$this->users[$i]["groups"][$groupname] = 0;
	return;
      }
    }
  }

  /**
   * Checks if a user is member of a group.
   *
   * @param $username Name of the user.
   * @param $groupname Name of the group.
   * @return 1 if the user is member of the group, 0 otherwise.
   */
  function isMember($username, $groupname) {
    for ($i = 0; $i < count($this->users); $i++) {
      if ($this->users[$i]["username"] == $username) {
	return $this->users[$i]["groups"][$groupname];
      }
    }
    return 0;
  }

  /**
   * Returns a string representation of this object.
   */
  function toString() {
    $s = "";
    $users = $this->getUsers();
    for ($i = 0; $i < count($users); $i++) {
      $s .= sprintf("%s: ", $users[$i]);
      $groups = $this->getUserGroups($users[$i]);
      $s .= implode(", ", $groups);
      $s .= "\n";
    }
    return $s;
  }
}

?>
